package MT::Plugin::SKR::EasyAreaTargeting;

use strict;
use MT 4;
use MT::Template;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'EasyAreaTargeting';
$VERSION = '0.01';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
    name => $MYNAME,
    id => lc $MYNAME,
    key => lc $MYNAME,
    version => $VERSION,
    author_name => 'SKYARC System Co.,Ltd.',
    author_link => 'http://www.skyarc.co.jp/',
    plugin_link => 'http://www.skyarc.co.jp/engineerblog/entry/easyareatargeting.html',
    doc_link => 'http://code.google.com/intl/ja/apis/ajax/documentation/',
    description => <<HTMLHEREDOC,
Enable you to support the area targeting contents powered by Google FREELY.
HTMLHEREDOC
        system_config_template => 'tmpl/config.tmpl',
        settings => new MT::PluginSettings([
            ['apikey', { Default => undef, Scope => 'system' }],
        ]),
});
MT->add_plugin( $plugin );

sub instance { $plugin; }



### Registry
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        tags => {
            function => {
                "Use$MYNAME" => \&_tag_UseEasyAreaTargeting,
                "AreaTargetedInclude" => \&_tag_AreaTargetedInclude,
            },
        },
        applications => {
            cms => {
                methods => {
                    $MYNAME => \&_hdlr_EasyAreaTargeting,
                },
            },
        },
    });
}



### Tag - MTUseEasyAreaTargeting
sub _tag_UseEasyAreaTargeting {
    my ($ctx, $args) = @_;

    $ctx->stash ($MYNAME. '::count', 0);

    my $apikey = &instance->get_config_value ('apikey')
        or return ''; # not yet setting
    return <<HTMLHEREDOC;
<script type="text/javascript" src="http://www.google.com/jsapi?key=$apikey"></script>
<script type="text/javascript">
   (typeof (\$) == 'undefined') && google.load ("jquery", "1.3.1");
</script>
HTMLHEREDOC
};

### Tag - MTAreaTargetedInclude
sub _tag_AreaTargetedInclude {
    my ($ctx, $args) = @_;

    my $name = $args->{name}
        or return $ctx->error ('name paramaeter must be required');

    my $url = $ctx->tag ('AdminCGIPath'). $ctx->tag ('AdminScript');

    my $blog_id = $ctx->stash('blog') && $ctx->stash('blog')->id
        || $args->{blog_id}
        || 0;

    my $count = $ctx->stash ($MYNAME. '::count');
    $ctx->stash ($MYNAME. '::count', ++$count);

    my $html = <<HTMLHEREDOC;
<script type="text/javascript" id="$MYNAME$count">
    jQuery.post ("$url", {
HTMLHEREDOC

    # Switch the mode
    if (defined (my $radius = $args->{radius})) {
        $html .= <<HTMLHEREDOC;
            radius: "$radius",
            lat: google.loader.ClientLocation ? google.loader.ClientLocation.latitude : 0,
            long: google.loader.ClientLocation ? google.loader.ClientLocation.longitude : 0,
HTMLHEREDOC
    }
    else {
        $html .= <<HTMLHEREDOC;
            country: google.loader.ClientLocation ? google.loader.ClientLocation.address.country : '',
            region: google.loader.ClientLocation ? google.loader.ClientLocation.address.region : '',
            city: google.loader.ClientLocation ? google.loader.ClientLocation.address.city : '',
HTMLHEREDOC
    }

    $html .= <<HTMLHEREDOC;
            __mode: "$MYNAME", name: "$name", blog_id: $blog_id }, function (html) {
        jQuery("#$MYNAME$count").after(html);
    });
</script>
HTMLHEREDOC
    $html;
};



### Method - EasyAreaTargeting
sub _hdlr_EasyAreaTargeting {
    my ($app) = @_;

    my $name = $app->param('name')
        or return '<!-- name parameter must be required -->'; # no content
    my $blog_id = $app->param('blog_id') || 0;

    # Retrieving the template which is close to the specified point.
    if (defined (my $radius = $app->param('radius'))) {
        my $lat = $app->param ('lat') || 0;
        my $long = $app->param ('long') || 0;
        my $iter = MT::Template->load_iter ({ blog_id => $blog_id });
        while (my $tmpl = $iter->()) {
            if ($tmpl->name =~ /^\Q$MYNAME\E \Q$name\E \(([0-9\.-]+),([0-9\.-]+)\)$/) {
                if (_get_distance ($lat,$long, $1,$2) <= $radius) {
                    return $tmpl->build || $tmpl->errstr;
                }
            }
        }
    }
    # Retrieving the template from the specified country, region and city
    else {
        my @params = ($app->param('country') || '', $app->param('region') || '', $app->param('city') || '', '(dummy)');
        do {
            pop @params;
            my $tmpl_name = join ' ', (
                $MYNAME,
                $name,
                join ('/', @params),
            );
            if (defined (my $tmpl = MT::Template->load ({ blog_id => $blog_id, name => $tmpl_name }))) {
                return $tmpl->build || $tmpl->errstr;
            }
        } while (@params);
    }
    return '<!-- template not found -->';
}

# Distance between the two latitudes and longitude
sub _get_distance {
    my ($lat1,$long1, $lat2,$long2) = @_;
    0; # not yet implemented
}

1;
__END__

<$MTAreaTargetedInclude name="hoge"$>
<$MTAreaTargetedInclude name="hoge" radius="100"$>
